/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

//import pl.edu.mimuw.bioputer.tav4sb.workflows.utils.StaticReader;
import java.io.IOException;
import java.net.MalformedURLException;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLStreamException;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.io.StringWriter;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;
import org.sbml.jsbml.*;


public class GetSbmlParams {
    static class ParamObj {
        String type;
        String name;
        BigDecimal value;
    }

    static String CONTEXT_SEP = "."; // not allowed in the SBML SId type
    static String KINETIC_PARTYPE = "kinetic";
    static String SPECIES_PARTYPE = "species";

    static ParamObj parameter(String type, String name, BigDecimal value) {
        ParamObj ret = new ParamObj();
        ret.type = type;
        ret.name = name;
        ret.value = value;

        return ret;
    }

    static ParamObj kineticParam(QuantityWithUnit param, Reaction reaction) {
        BigDecimal value = null;
        if (param.isSetValue())
            value = new BigDecimal(param.getValue());
        String name = param.getId();
        if (reaction != null)
            name = reaction.getId() + CONTEXT_SEP + name;
        return parameter(KINETIC_PARTYPE, name, value);
    }

    static ParamObj speciesParam(Species species) {
        BigDecimal value = null;
        if (species.isSetInitialConcentration()) {
            value = new BigDecimal(species.getInitialConcentration());
        } else if (species.isSetInitialAmount()) {
            value = new BigDecimal(species.getInitialAmount());
        }
        return parameter(SPECIES_PARTYPE, species.getId(), value);
    }

    static String PARAMS_TAGNAME = "paramList";
    static String PARAM_TAGNAME = "param";
    static String NAME_ATTRNAME = "name";
    static String TYPE_ATTRNAME = "type";

    static String toTag(List<ParamObj> paramlist) throws XMLStreamException,
            FactoryConfigurationError {
        StringWriter sw = new StringWriter();
        XMLStreamWriter xw = XMLOutputFactory.newInstance()
                .createXMLStreamWriter(sw);

        xw.writeStartDocument();
        xw.writeStartElement(PARAMS_TAGNAME);

        for (ParamObj paramobj : paramlist) {
            xw.writeStartElement(PARAM_TAGNAME);
            xw.writeAttribute(NAME_ATTRNAME, paramobj.name);
            xw.writeAttribute(TYPE_ATTRNAME, paramobj.type);
            if (paramobj.value != null)
                xw.writeCharacters(paramobj.value.toString());
            xw.writeEndElement();
        }

        xw.writeEndElement();
        xw.writeEndDocument();

        return sw.toString();
    }

    /**
     * @param args
     * @throws XMLStreamException
     * @throws IOException
     * @throws MalformedURLException
     */
    public static void main(String[] args) throws XMLStreamException,
            MalformedURLException, IOException {

//        String model = StaticReader
//                .readUrl("http://localhost/tav4sb/models/enzyme.xml");
        String model = ""+
            "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">\n"+
            "    <model id=\"Reaction\" name=\"Single reaction\">\n"+
            "    <listOfUnitDefinitions>\n"+
            "      <unitDefinition id=\"substance\" name=\"moles\">\n"+
            "        <listOfUnits>\n"+
            "          <unit kind=\"mole\" scale=\"1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "        </listOfUnits>\n"+
            "      </unitDefinition>\n"+
            "      <unitDefinition id=\"M\" name=\"M\">\n"+
            "        <listOfUnits>\n"+
            "          <unit kind=\"mole\" scale=\"1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "          <unit kind=\"litre\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "        </listOfUnits>\n"+
            "      </unitDefinition>\n"+
            "      <unitDefinition id=\"M_per_second\" name=\"M/s\">\n"+
            "        <listOfUnits>\n"+
            "          <unit kind=\"mole\" scale=\"1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "          <unit kind=\"litre\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "          <unit kind=\"second\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "        </listOfUnits>\n"+
            "      </unitDefinition>\n"+
            "      <unitDefinition id=\"sec_inv\" name=\"1/s\">\n"+
            "        <listOfUnits>\n"+
            "          <unit kind=\"second\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
            "        </listOfUnits>\n"+
            "      </unitDefinition>\n"+
            "    </listOfUnitDefinitions>\n"+
            "    <listOfCompartments>\n"+
            "      <compartment id=\"s\" name=\"solution\" size=\"1\"/>\n"+
            "    </listOfCompartments>\n"+
            "    <listOfSpecies>\n"+
            "      <species id=\"S\" name=\"Substrate\" compartment=\"s\" initialConcentration=\"10\"/>\n"+
            "      <species id=\"P\" name=\"Product\" compartment=\"s\" initialConcentration=\"0\"/>\n"+
            "    </listOfSpecies>\n"+
            "    <listOfReactions>\n"+
            "      <reaction id=\"r1\" name=\"Substrate conversion\" reversible=\"false\">\n"+
            "        <listOfReactants>\n"+
            "          <speciesReference species=\"S\"/>\n"+
            "        </listOfReactants>\n"+
            "        <listOfProducts>\n"+
            "          <speciesReference species=\"P\"/>\n"+
            "        </listOfProducts>\n"+
            "        <kineticLaw>\n"+
            "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">\n"+
            "            <apply>\n"+
            "              <times/>\n"+
            "              <ci> k </ci>\n"+
            "              <ci> S </ci>\n"+
            "            </apply>\n"+
            "          </math>\n"+
            "          <listOfParameters>\n"+
            "            <parameter id=\"k\" value=\"0.1\"/>\n"+
            "          </listOfParameters>\n"+
            "        </kineticLaw>\n"+
            "      </reaction>\n"+
            "    </listOfReactions>\n"+
            "  </model>\n"+
            "</sbml>";

        SBMLReader reader = new SBMLReader();
        SBMLDocument doc = reader.readSBMLFromString(model);
        Model m = doc.getModel();

        List<ParamObj> params = new ArrayList<ParamObj>();
        // local kinetic parameters
        for (int i = 0; i < m.getNumReactions(); i++) {
            Reaction reaction = m.getReaction(i);
            KineticLaw kineticLaw = reaction.getKineticLaw();
            for (int j = 0; j < kineticLaw.getNumLocalParameters(); j++) {
                params.add(kineticParam(kineticLaw.getLocalParameter(j), reaction));
            }
        }
        // global kinetic parameters
        for (int i = 0; i < m.getNumParameters(); i++) {
            params.add(kineticParam(m.getParameter(i), null));
        }
        // species
        for (int i = 0; i < m.getNumSpecies(); i++) {
            params.add(speciesParam(m.getSpecies(i)));
        }

        String paramListXml = toTag(params);

        System.out.println(paramListXml);
    }
}
