/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

import java.math.BigDecimal;
import java.math.MathContext;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MeanSquaredError {


    private static List<BigDecimal> parsePoint(List<String> pointStrList, int size) {
        List<BigDecimal> pointNumList = new ArrayList<BigDecimal>(size);
        for (int i = 0; i < size; i++) // trim higher dims
            pointNumList.add(i, new BigDecimal(pointStrList.get(i)));
        return pointNumList;
    }

    private static Map<List<BigDecimal>, List<BigDecimal>> argValList2Map(
            List<List<List<String>>> argValList,
            List<List<String>> argValTemplate) {
        int nArgs = argValTemplate.get(0).size();
        int nVals = argValTemplate.get(1).size();
        Map<List<BigDecimal>, List<BigDecimal>> argValMap = new HashMap<List<BigDecimal>,List<BigDecimal>>(argValList.size());
        for (List<List<String>> argValPair : argValList) {
            try {
                argValMap.put(
                        parsePoint(argValPair.get(0), nArgs),
                        parsePoint(argValPair.get(1), nVals)
                );
            } catch (IndexOutOfBoundsException pass) {} // ignore invalid dims
        }
        return argValMap;
    }

    private static BigDecimal arithmeticMean(List<BigDecimal> nums) {
        BigDecimal ret = new BigDecimal(0);
        for (BigDecimal num : nums)
            ret = ret.add(num);
        return ret.divide(new BigDecimal(nums.size()), MathContext.DECIMAL128);
    }

    public static void main(String[] args) {

        List<List<List<String>>> argValList1 = new ArrayList<List<List<String>>>(4);
        argValList1.add(createDataListPair( //important if argValTemplate is not given
                Arrays.asList(new String[] {"0"}),
                Arrays.asList(new String[] {"0", "0", "0"})
        ));
        argValList1.add(createDataListPair( //invalid: should be ignored
                Arrays.asList(new String[] {"0", "0"}),
                Arrays.asList(new String[] {"0"})
        ));
        argValList1.add(createDataListPair(
                Arrays.asList(new String[] {"1", "0"}), //valid: should be trimmed
                Arrays.asList(new String[] {"0", "0", "0"})
        ));
        List<List<List<String>>> argValList2 = new ArrayList<List<List<String>>>(2);
        argValList2.add(createDataListPair(
                Arrays.asList(new String[] {"0"}),
                Arrays.asList(new String[] {"0", "1", "0"})
        ));
        argValList2.add(createDataListPair(
                Arrays.asList(new String[] {"1"}),
                Arrays.asList(new String[] {"1", "2", "0"})
        ));
        argValList2.add(createDataListPair(
                Arrays.asList(new String[] {"2",}), //valid but w/o corresponding element: should be ignored
                Arrays.asList(new String[] {"0", "0", "0"})
        ));
        List<List<String>> argValTemplate = null;


        BigDecimal error;
        if (argValList1.size() == 0 || argValList2.size() == 0)
            error = new BigDecimal(0);
        else {
            if (argValTemplate == null)
                argValTemplate = argValList1.get(0);
            if (argValTemplate.size() < 2 || argValTemplate.get(0).size() < 1 || argValTemplate.get(1).size() < 1) {
                throw new RuntimeException("Invalid format of the point template; expected pair of lists, each with size of at least 1.");
            }
            Map<List<BigDecimal>, List<BigDecimal>> argValMap1 = argValList2Map(argValList1, argValTemplate);
            Map<List<BigDecimal>, List<BigDecimal>> argValMap2 = argValList2Map(argValList2, argValTemplate);

            // If data lists have different arguments sets, only intersecting data points will be considered.
            Set<List<BigDecimal>> argSet = new HashSet<List<BigDecimal>>(argValMap1.keySet());
            argSet.retainAll(argValMap2.keySet());
            int N = argValTemplate.get(1).size(); // inner sum limit: number of values
            int M = argSet.size(); // outer sum limit: number of data points
            List<BigDecimal> errorList = new ArrayList<BigDecimal>(M);
            for (List<BigDecimal> arg: argSet) {
                List<BigDecimal> residuals = new ArrayList<BigDecimal>(N);
                List<BigDecimal> val1 = argValMap1.get(arg);
                List<BigDecimal> val2 = argValMap2.get(arg);
                for (int i = 0; i < N; i++)
                    residuals.add(val1.get(i).subtract(val2.get(i)).pow(2));
                errorList.add(arithmeticMean(residuals));
            }
            error = arithmeticMean(errorList);
        }

        System.out.println(error);
    }

    private static List<List<String>> createDataListPair(List<String> args,
            List<String> vals) {
        ArrayList<List<String>> ret = new ArrayList<List<String>>(2);
        ret.add(args);
        ret.add(vals);
        return ret;
    }
}
