/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

class MathType {
    String type;
    Object value;
}
//class MathString extends MathType {
//    final String type = "String";
//    String value;
//}
//class MathNumber extends MathType {
//    final String type = "Number";
//    BigDecimal value;
//}
//class MathSymbol extends MathType {
//    final String type = "Symbol";
//    String value;
//}
//class MathList extends MathType {
//    final String type = "List";
//    List<MathType> value;
//}

class PartiallyParsedListObj {
    MathType head;
    String tail;
}

class OptionObj {
    String name;
    MathType math;
}

class ExceptionMessage extends RuntimeException {
    public ExceptionMessage(String msg) {
        super(msg);
    }
}

class OptionFormatException extends Exception {
    public OptionFormatException(String option, String msg) {
        super(option + (msg == null ? "": ("; " + msg)) + ".");
    }
}

public class SetOptionXmlList {

    static String NAME_VALUE_SEP = "->";
    static String STRING_DELIM = "\"";
    static String LIST_START = "{";
    static String LIST_END = "}";
    static String LIST_EL_SEP = ",";
    static String ESCAPE_CHAR = "\\";

    static String mathStringUnescape(String mathStringStr) {
        // string delimiter escaping is optional
        return mathStringStr.replace(ESCAPE_CHAR+LIST_EL_SEP, LIST_EL_SEP)
            .replace(ESCAPE_CHAR+LIST_START, LIST_START)
            .replace(ESCAPE_CHAR+LIST_END, LIST_END)
            .replace(ESCAPE_CHAR+STRING_DELIM, STRING_DELIM);
    }

    /**
     * Same as the String method indexOf, but returns the length of the string
     * param instead of -1, if the separator wasn't found.
     */
    private static int indexOf(String str, String sep) {
        int ret = str.indexOf(sep);
        return (ret < 0) ? str.length() : ret;
    }

    /**
     * Get the first index of a non-escaped sep in str. If not found returns the
     * length of the string which is not a valid index but is a valid arg for
     * the substring method.
     */
    private static int unescapedIndexOf(String str, String sep) {
        // we're interested in non-escaped separators
        if (str.startsWith(sep))
            return 0;
        int i = -1; // sep start index
        while (i < str.length()-1) {
            i = str.indexOf(sep, i+1);
            if (i < 0)
                i = str.length();
            else if (!str.startsWith(ESCAPE_CHAR, i-1))
                break;
        }
        return i;
    }

    static PartiallyParsedListObj partiallyParsedList(String tail) {
        MathType head = new MathType();
        if (tail.startsWith(LIST_START)) {
            // extract inner list element
            tail = tail.substring(1); // cut the opening bracket
            PartiallyParsedListObj ppInnerList;
            List<MathType> innerList = new LinkedList<MathType>();
            while (!tail.startsWith(LIST_END)) { // stop at the closing bracket
                ppInnerList = partiallyParsedList(tail);
                innerList.add(ppInnerList.head);
                tail = ppInnerList.tail;
                if (tail.length() == 0) {
                    throw new ExceptionMessage("missing closing list bracket '}'");
                }
            }
            tail = tail.substring(1).trim(); // cut the closing bracket
            head.type = "List";
            head.value = innerList;
        } else {
            // extract basic type element
            int splitIndex;
            if (tail.startsWith(STRING_DELIM)) {
                // extract element by jumping to the end of the string delimiter
                splitIndex = Math.min(unescapedIndexOf(tail.substring(1),
                        STRING_DELIM) + 2, tail.length());
            } else {
                // extract element by splitting tail with a non-escaped comma or
                // a closing bracket
                splitIndex = Math.min(indexOf(tail, LIST_EL_SEP),
                        indexOf(tail, LIST_END));
            }
            head = mathtype(tail.substring(0, splitIndex));
            tail = tail.substring(splitIndex).trim();
        }
        // if necessary, cut the list element separator
        if (tail.length() > 0 && !tail.startsWith(LIST_END)) {
            if (tail.startsWith(LIST_EL_SEP)) {
                tail = tail.substring(1).trim();
            } else {
                throw new ExceptionMessage("missing list element separator ',' near '" + tail.substring(0, Math.min(10, tail.length())) + "...'");
            }
        }

        PartiallyParsedListObj ret = new PartiallyParsedListObj();
        ret.head = head;
        ret.tail = tail;
        return ret;
    }

    static MathType mathtype(String mathtypestr) {
        String in = mathtypestr.trim();
        String type = null;
        Object value = null;
        if (in.startsWith(STRING_DELIM)) {
            if (!(in.length()>1 && in.endsWith(STRING_DELIM))) {
                throw new ExceptionMessage("missing closing string quote '\"'");
            }
            type = "String";
            value = mathStringUnescape(in.substring(1, in.length()-1));
        } else if (in.startsWith(LIST_START)) {
            if (!in.endsWith(LIST_END)) {
                throw new ExceptionMessage("missing closing list bracket '}'");
            }
            type = "List";
            List<MathType> list = new LinkedList<MathType>();
            // manual parsing cuz of nested lists
            PartiallyParsedListObj partialyParsedList;
            in = in.substring(1, in.length()-1).trim();
            while (in.length()>0) {
                partialyParsedList = partiallyParsedList(in);
                in = partialyParsedList.tail;
                list.add(partialyParsedList.head);
            }
            value = list;
        } else if (in.contains(".")) {
            type = "Number";
            try {
                value = new BigDecimal(in);
            } catch (NumberFormatException e) {
                throw new ExceptionMessage("number '" + in + "' format exception");
            }
        } else {
            type = "Symbol";
            value = in;
        }

        MathType ret = new MathType();
        ret.type = type;
        ret.value = value;
        return ret;
    }

    static OptionObj option(String optionstr) throws OptionFormatException {
        String[] nameAndValue = optionstr.split("->", 2);
        String name = nameAndValue[0].trim();
        if (nameAndValue.length < 2) {
            throw new OptionFormatException(optionstr, "missing value");
        }
        MathType math = null;
        try {
            math = mathtype(nameAndValue[1]);
        } catch (ExceptionMessage e) {
            throw new OptionFormatException(optionstr, e.getMessage());
        }

        OptionObj ret = new OptionObj();
        ret.name = name;
        ret.math = math;
        return ret;
    }

    static String XSI_NS_PREFIX = "xsi";
    static String XSI_NS_URI = "http://www.w3.org/2001/XMLSchema-instance";
    static String MATH_NS_PREFIX = "math";
    static String MATH_NS_URI = "http://bioputer.mimuw.edu.pl/tav4sb/services/xsd/mathematica";
    static String OPTION_TAGNAME = "option";
    static String NAME_TAGNAME = "name";
    static String VALUE_TAGNAME = "value";
    static String EL_TAGNAME = "element";

    private static void writeMathTypeElement(XMLStreamWriter xw, MathType mathtype, String elname) throws XMLStreamException {
        xw.writeStartElement(elname);
        xw.writeAttribute(XSI_NS_PREFIX, XSI_NS_URI, "type", MATH_NS_PREFIX+":"+mathtype.type);
        if (mathtype.type.equals("List")) {
            for (MathType elmathtype: (List<MathType>) mathtype.value) {
                writeMathTypeElement(xw, elmathtype, EL_TAGNAME);
            }
        } else {
            xw.writeStartElement(mathtype.type.toLowerCase());
            xw.writeCharacters(mathtype.value.toString());
            xw.writeEndElement();
        }
        xw.writeEndElement();
    }

    private static String toTag(OptionObj opt) throws XMLStreamException, FactoryConfigurationError {
        StringWriter sw = new StringWriter();
        XMLStreamWriter xw = XMLOutputFactory.newInstance().createXMLStreamWriter(sw);

        xw.writeStartDocument();
        xw.writeStartElement(OPTION_TAGNAME);
        xw.writeNamespace(XSI_NS_PREFIX, XSI_NS_URI);
        xw.writeNamespace(MATH_NS_PREFIX, MATH_NS_URI);
        xw.writeStartElement(NAME_TAGNAME);
        xw.writeCharacters(opt.name);
        xw.writeEndElement();
        writeMathTypeElement(xw, opt.math, VALUE_TAGNAME);
        xw.writeEndDocument();

        String ret = sw.toString();

        System.out.println(ret);
        return ret;
    }

    public static void main(String[] args) throws XMLStreamException, FactoryConfigurationError, OptionFormatException {
        List<String> input = Arrays.asList(new String[]{
                "AxesLabel ->{\"t [\\\"s\\\"]\", \"u(0)\", \"f(t, u(0),)\" }",
                "PlotLabel -> \"Function f of time t and initial condition u(0) for multiplier 1/2, 1, and 2.\"",
                "PlotLegend-> None",
                "MarkerColor -> { {0.5, 0.5, 0.5}, Red, Gray }",
                "OpacityLevel -> { 0.2, 1., 0.2 }",
                "test -> { {.0,\"}\\}\",{0.}}, sym }",
        });

        List<String> output = new LinkedList<String>();
        if (!input.isEmpty()) {
            for (String optionstr: input) {
                output.add(toTag(option(optionstr)));
            }
        }
    }
}
