/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;

import java.io.ByteArrayInputStream;
import java.math.BigDecimal;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.sbml.jsbml.SBMLReader;
import org.sbml.jsbml.SBMLWriter;
import org.w3c.dom.NodeList;



import java.io.IOException;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.stream.XMLStreamException;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.sbml.jsbml.Model;
import org.sbml.jsbml.QuantityWithUnit;
import org.sbml.jsbml.SBMLException;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;


public class SetSbmlParams {
    static class ParamObj {
        String type;
        String name;
        BigDecimal value;
    }

    static ParamObj parameter(String type, String name, BigDecimal value) {
        ParamObj ret = new ParamObj();
        ret.type = type;
        ret.name = name;
        ret.value = value;

        return ret;
    }

    static String KINETIC_PARTYPE = "kinetic";
    static String SPECIES_PARTYPE = "species";

    private static void setParamValue(Model m, ParamObj paramobj) {
        QuantityWithUnit param;
        if (paramobj.type.equals(SPECIES_PARTYPE)) {
            param = m.getSpecies(paramobj.name);
        } else if (paramobj.type.equals(KINETIC_PARTYPE)) {
            String rId = null;
            String pId = paramobj.name;
            String[] rAndPId = pId.split("\\.");
            if (rAndPId.length > 2) {
                throw new RuntimeException("Wrong parameter name format: \""
                        + paramobj.name + "\".");
            } else if (rAndPId.length > 1) {
                rId = rAndPId[0];
                pId = rAndPId[1];
            }
            param = (rId == null) ? m.getParameter(pId) : m.getReaction(rId).getKineticLaw().getLocalParameter(pId);

        } else {
            throw new RuntimeException("Wrong parameter type format: \""
                    + paramobj.type + "\".");
        }
        param.setValue(paramobj.value.doubleValue());
    }

    static String XML_OUTPUT_FACTORY_PROPKEY = "javax.xml.stream.XMLOutputFactory";
    static String XML_OUTPUT_FACTORY_PROPVAL = "com.ctc.wstx.stax.WstxOutputFactory";

    private static String modelToString(Model m) throws XMLStreamException, SBMLException {
        // ATTENTION: requires the system classloader persistence of the referring beanshell processor
        String ret = null;
        String propBkp = System.getProperty(XML_OUTPUT_FACTORY_PROPKEY);
        boolean isDiffProp = !XML_OUTPUT_FACTORY_PROPVAL.equals(propBkp);
        try {
            if (isDiffProp)
                System.setProperty(XML_OUTPUT_FACTORY_PROPKEY, XML_OUTPUT_FACTORY_PROPVAL);
            ret = new SBMLWriter().writeSBMLToString(m.getSBMLDocument());
        } finally {
            if (propBkp != null) {
                if (isDiffProp)
                    System.setProperty(XML_OUTPUT_FACTORY_PROPKEY, propBkp);
            } else
                System.getProperties().remove(XML_OUTPUT_FACTORY_PROPKEY);
        }
        return ret;
    }

    static String PARAM_TAGNAME = "param";
    static String NAME_ATTRNAME = "name";
    static String TYPE_ATTRNAME = "type";

    public static void main(String[] args) throws SAXException, IOException, ParserConfigurationException, XMLStreamException, XPathExpressionException, SBMLException {
        String modelInXml = ""+
        "<sbml xmlns=\"http://www.sbml.org/sbml/level2\" level=\"2\" version=\"1\">\n"+
        "    <model id=\"Reaction\" name=\"Single reaction\">\n"+
        "    <listOfUnitDefinitions>\n"+
        "      <unitDefinition id=\"substance\" name=\"moles\">\n"+
        "        <listOfUnits>\n"+
        "          <unit kind=\"mole\" scale=\"1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "        </listOfUnits>\n"+
        "      </unitDefinition>\n"+
        "      <unitDefinition id=\"M\" name=\"M\">\n"+
        "        <listOfUnits>\n"+
        "          <unit kind=\"mole\" scale=\"1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "          <unit kind=\"litre\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "        </listOfUnits>\n"+
        "      </unitDefinition>\n"+
        "      <unitDefinition id=\"M_per_second\" name=\"M/s\">\n"+
        "        <listOfUnits>\n"+
        "          <unit kind=\"mole\" scale=\"1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "          <unit kind=\"litre\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "          <unit kind=\"second\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "        </listOfUnits>\n"+
        "      </unitDefinition>\n"+
        "      <unitDefinition id=\"sec_inv\" name=\"1/s\">\n"+
        "        <listOfUnits>\n"+
        "          <unit kind=\"second\" exponent=\"-1\" multiplier=\"1\" offset=\"0\"/>\n"+
        "        </listOfUnits>\n"+
        "      </unitDefinition>\n"+
        "    </listOfUnitDefinitions>\n"+
        "    <listOfCompartments>\n"+
        "      <compartment id=\"s\" name=\"solution\" size=\"1\"/>\n"+
        "    </listOfCompartments>\n"+
        "    <listOfSpecies>\n"+
        "      <species id=\"S\" name=\"Substrate\" compartment=\"s\" initialConcentration=\"10\"/>\n"+
        "      <species id=\"P\" name=\"Product\" compartment=\"s\" initialConcentration=\"0\"/>\n"+
        "    </listOfSpecies>\n"+
        "    <listOfReactions>\n"+
        "      <reaction id=\"r1\" name=\"Substrate conversion\" reversible=\"false\">\n"+
        "        <listOfReactants>\n"+
        "          <speciesReference species=\"S\"/>\n"+
        "        </listOfReactants>\n"+
        "        <listOfProducts>\n"+
        "          <speciesReference species=\"P\"/>\n"+
        "        </listOfProducts>\n"+
        "        <kineticLaw>\n"+
        "          <math xmlns=\"http://www.w3.org/1998/Math/MathML\">\n"+
        "            <apply>\n"+
        "              <times/>\n"+
        "              <ci> k </ci>\n"+
        "              <ci> S </ci>\n"+
        "            </apply>\n"+
        "          </math>\n"+
        "          <listOfParameters>\n"+
        "            <parameter id=\"k\" value=\"0.1\"/>\n"+
        "          </listOfParameters>\n"+
        "        </kineticLaw>\n"+
        "      </reaction>\n"+
        "    </listOfReactions>\n"+
        "  </model>\n"+
        "</sbml>";
        String paramListXml = ""+
        "<whatever>"+
        "   <param name=\"r1.k\" type=\"kinetic\">0.2</param>"+
        "   <param name=\"S\" type=\"species\">5</param>"+
        "</whatever>";


        Document pDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(paramListXml.getBytes()));
        XPathExpression xpexpr = XPathFactory.newInstance().newXPath().compile("/*/" + PARAM_TAGNAME);
        NodeList paramNodes = (NodeList) xpexpr.evaluate(pDoc, XPathConstants.NODESET);
        List<ParamObj> params = new ArrayList<ParamObj>(paramNodes.getLength());
        for (int i = 0; i < paramNodes.getLength(); i++) {
            Node paramNode = paramNodes.item(i);
            NamedNodeMap paramAttrs = paramNode.getAttributes();
            params.add(parameter(
                    paramAttrs.getNamedItem(TYPE_ATTRNAME).getNodeValue(),
                    paramAttrs.getNamedItem(NAME_ATTRNAME).getNodeValue(),
                    new BigDecimal(paramNode.getTextContent())
            ));
        }

        Model m = new SBMLReader().readSBMLFromString(modelInXml).getModel();
        for (ParamObj param : params) {
            setParamValue(m, param);
        }

        String modelOutXml = modelToString(m);
        System.out.println(modelOutXml);
    }
}
