/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica;

import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.TaskConsumerManager;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils.MathematicaProperties;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.CalcSensitivityWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.ClassifySamplesWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.MathPlotWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.PlotTrajectories3DWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.PlotTrajectoriesWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.SampleParamsWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.SelectObservablesWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.worker.SelectThresholdWorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.CommandLineArgs;
import pl.edu.mimuw.bioputer.tav4sb.utils.cli.IncorrectArgumentFormatException;

/**
 *
 * @author michal
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class);
    private static final MathematicaProperties props = new MathematicaProperties();
    // TODO: mv multiple-tasks names to the MathematicaProperties class? (it's DRY, only not in same place as other props)
    public static final String SELECT_THRESHOLD = "selectThreshold";
    public static final String MATH_PLOT = "mathPlot";
    public static final String CALC_SENSITIVITY = "calcSensitivity";
    public static final String CLASSIFY_SAMPLES = "classifySamples";
    public static final String PLOT_TRAJECTORIES = "plotTrajectories";
    public static final String PLOT_TRAJECTORIES3D = "plotTrajectories3D";
    public static final String SAMPLE_PARAMS = "sampleParams";
    public static final String SELECT_OBSERVABLES = "selectObservables";

    public static void main(String[] args) {
        try {
            // get arguments
            CommandLineArgs command = new CommandLineArgs(args);
            String brokerURL = command.getArgValue(props.BROKER_URL, props.getBrokerUrl());
            String queueName = command.getArgValue(props.QUEUE_NAME, props.getQueueName());
            String threadNumber = command.getArgValue(props.THREAD_NUMBER, props.getThreadNumber());
            String selectThreshold = command.getArgValue(SELECT_THRESHOLD, props.getTaskType(SELECT_THRESHOLD));
            String mathPlot = command.getArgValue(MATH_PLOT, props.getTaskType(MATH_PLOT));
            String calcSensitivity = command.getArgValue(CALC_SENSITIVITY, props.getTaskType(CALC_SENSITIVITY));
            String classifySamples = command.getArgValue(CLASSIFY_SAMPLES, props.getTaskType(CLASSIFY_SAMPLES));
            String plotTrajectories = command.getArgValue(PLOT_TRAJECTORIES, props.getTaskType(PLOT_TRAJECTORIES));
            String plotTrajectories3D = command.getArgValue(PLOT_TRAJECTORIES3D, props.getTaskType(PLOT_TRAJECTORIES3D));
            String sampleParams = command.getArgValue(SAMPLE_PARAMS, props.getTaskType(SAMPLE_PARAMS));
            String selectObservables = command.getArgValue(SELECT_OBSERVABLES, props.getTaskType(SELECT_OBSERVABLES));
            if (command.isHelpSet()) {
                command.printHelp(System.out);
                return;
            }
            // log argument
            logger.info("Starting ...");
            logger.info("Broker URL: " + brokerURL);
            logger.info("Queue name: " + queueName);
            logger.info("Thread number: " + threadNumber);
            logger.info("Select threshold task: " + selectThreshold);
            logger.info("Calculate sensitivity task: " + calcSensitivity);
            logger.info("Classify samples task: " + classifySamples);
            logger.info("Plot trajectories task: " + plotTrajectories);
            logger.info("Plot trajectories 3D task: " + plotTrajectories3D);
            logger.info("Sample params task: " + sampleParams);
            logger.info("Select observables task: " + selectObservables);
            // register workers
            TaskConsumerManager consumer = new TaskConsumerManager(brokerURL, queueName);
            consumer.registerFactory(selectThreshold, SelectThresholdWorkerFactory.getInstance());
            consumer.registerFactory(mathPlot, MathPlotWorkerFactory.getInstance());
            consumer.registerFactory(calcSensitivity, CalcSensitivityWorkerFactory.getInstance());
            consumer.registerFactory(classifySamples, ClassifySamplesWorkerFactory.getInstance());
            consumer.registerFactory(plotTrajectories, PlotTrajectoriesWorkerFactory.getInstance());
            consumer.registerFactory(plotTrajectories3D, PlotTrajectories3DWorkerFactory.getInstance());
            consumer.registerFactory(sampleParams, SampleParamsWorkerFactory.getInstance());
            consumer.registerFactory(selectObservables, SelectObservablesWorkerFactory.getInstance());
            // start consumer
            consumer.run(Integer.parseInt(threadNumber));
        } catch (IncorrectArgumentFormatException e) {
            logger.error("Incorrect argument format");
            System.exit(1);
        } catch (Exception e) {
            logger.error("Exception occured: " + e.getMessage(), e);
            System.exit(1);
        }
    }
}
