/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc;

import com.wolfram.jlink.MathLinkException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.apache.log4j.Logger;

import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngine;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineException;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineManager;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils.XsdUtils;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.NewArrayOfParam;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.NewParam;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.Param;

/**
 *
 * @author michal
 */
public class SampleParams {

    static final String[] sourceFiles = {"SampleParams.m"};
    private static final Logger logger = Logger.getLogger(SampleParams.class);
    private static final String sampleParams = "sampleParams";
    private static final String checkParams = "checkParams";
    public static final String UNIFORM = "uniform";
    public static final String LOG_SCALE = "LOG";
    public static final String MONTECARLO = "MC";
    public static final String LHS = "LHS";
    public static final String MC_CARTESIAN = "MC_CARTESIAN";

    static public List<NewArrayOfParam> sampleParams(String distribution, String scale, BigDecimal rangeMultiplier, String method, int nr,
            List<Param> params) throws MathLinkException, MEngineException {
        //TODO: pass List<NewParam> as input
        logger.debug(sampleParams);

        /* Check and prepare parameters */
        if (rangeMultiplier == null) {
            logger.debug(sampleParams + ": rangeMultiplier is null - using default value: 1");
            rangeMultiplier = BigDecimal.ONE;
        }
        checkParams(distribution, scale, rangeMultiplier, method, nr, params);

        BigDecimal[] lowerBounds = XsdUtils.extractParamLowerBounds(params);
        BigDecimal[] upperBounds = XsdUtils.extractParamUpperBounds(params);
        BigDecimal[] values = XsdUtils.extractParamValues(params);

        MEngine engine = null;
        try {
            /* Acquire Mathematica engine */
            engine = MEngineManager.getEngine();
            engine.loadScriptFiles(sourceFiles);

            /* Execute function */
            engine.evaluateFunction("SampleParams", distribution, scale, method, nr, lowerBounds, upperBounds, values, rangeMultiplier);

            /* Get result */
            double[][] samples = engine.getDoubleDoubleArray();
            //TODO: add check: samples.length == nr

            /* Build response */
            List<NewArrayOfParam> res = new ArrayList<NewArrayOfParam>(samples.length);
            for (int i = 0; i < samples.length; i++) {
                logger.debug(sampleParams + ": sample #" + i +"=" + Arrays.toString(samples[i]));
                NewArrayOfParam aop = new NewArrayOfParam();
                aop.getParam().addAll(updateParams(params, samples[i]));
                res.add(aop);
            }
            return res;
        } catch (MathLinkException mle) {
            logger.error(sampleParams, mle);
            throw mle;
        } finally {
            if (engine != null) {
                logger.debug(sampleParams + ": relasing engine");
                engine.release();
            }
        }
    }

    private static List<NewParam> updateParams(List<Param> params,
            double[] samples) {
        List<NewParam> res = new ArrayList<NewParam>(samples.length);
        for (int i = 0; i < samples.length; i++) {
            NewParam np = XsdUtils.param2NewParam(params.get(i));
            np.setValue(new BigDecimal(samples[i]));
            res.add(np);
        }
        return res;
    }

    private static void raiseException(String message) {
        logger.debug(checkParams + ": " + message);
        throw new IllegalArgumentException(message);
    }

    private static void checkParams(String distribution, String scale, BigDecimal rangeMultiplier,
            String method, int nr, List<Param> params) {
        if (distribution == null) {
            raiseException("distribution is null");
        }
        if (scale == null) {
            raiseException("scale is null");
        }
        if (rangeMultiplier == null) {
            raiseException("rangeMultiplier is null");
        }
        if (method == null) {
            raiseException("method is null");
        }
        if (params == null) {
            raiseException("params list is null");
        }
        if (!distribution.equals(SampleParams.UNIFORM)) {
            raiseException("unsupported distribution type");
        }
        if ((!method.equals(SampleParams.MONTECARLO)) &&
                (!method.equals(SampleParams.LHS)) &&
                (!method.equals(SampleParams.MC_CARTESIAN))) {
            raiseException("unsupported method type");
        }
        if (nr <= 0) {
            raiseException("number of samples less or equal 0");
        }
        if (params.isEmpty()) {
            raiseException("empty params list");
        }
        for (Param param : params) {
            if (param == null) {
                raiseException("one of params is null");
            }
            if (param.getLowerBound() == null) {
                raiseException("one of params has not set lower bound");
            }
            if (param.getUpperBound() == null) {
                raiseException("one of params has not set upper bound");
            }
            if (param.getValue() == null) {
                raiseException("one of params has not set value");
            }
            if (param.getUpperBound().compareTo(param.getValue()) < 0) {
                raiseException("one of params has upper bound less than value");
            }
            if (param.getValue().compareTo(param.getLowerBound()) < 0) {
                raiseException("one of params has value less than lower bound");
            }
        }

    }
}
