(*******************************************************************************
*Copyright (C) 2011 Mikołaj Rybiński
*
*    This file is part of Tav4SB.
*
*    Tav4SB is free software: you can redistribute it and/or modify it under the
*    terms of the GNU Affero General Public License as published by the Free
*    Software Foundation, either version 3 of the License, or (at your option)
*    any later version.
*
*    Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
*    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
*    FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
*    more details.
*
*    You should have received a copy of the GNU Affero General Public License
*    along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
*
*    Contributors:
*        Paweł Banasik - initial implementation
*        Michał Lula - grid implementation
*        Mikołaj Rybiński - implementation and maintenance
*******************************************************************************)
(* Mathematica Package *)
BeginPackage["Utilz`"]
(* Exported symbols added here with SymbolName::usage *)
(********************************** LIST **************************************)
PlusList::usage = "PlusList[list]
    gives sum of the elements of the list as applied by the Plus function."
Pairs::usage = "Pairs[list,list]
    gives list of pairs of coresponding elements in both lists. Argument lists
    have to be of equal length."
PadRows::usage = "PadRows[{list, list, ...}]
    cyclically pads lists (array rows) on the input list (array) to the right,
    to the length of the longest list.
    "
ZipLists::usage = "ZipLists[list1,list2,...], ZipLists[{list1,list2,...}]
    gives list of tuples, where each tuple contains the i-th element from each
    of the argument lists. Argument lists are first cyclically padded right to
    have a equal length."
SingletonOrId::usage = "SingletonOrId[x]
    returns identity if x is a List, list with x as the only element (singleton)
    otherwise.
    "

(********************************** TEST **************************************)
PositiveIntegerQ::usage = "PositiveIntegerQ[expr]
    gives True if expr is a positive integer, and False otherwise.

    See IntegerQ for details.
    "
NonNegativeIntegerQ::usage = "NonNegativeIntegerQ[expr]
    gives True if expr is a non negative integer, and False otherwise.

    See IntegerQ for details.
    "
PositiveNumberQ::usage = "PositiveNumberQ[expr]
    gives True if expr is a positive number, and False otherwise.

    See NumberQ for details.
    "
NonNegativeNumberQ::usage = "NonNegativeNumberQ[expr]
    gives True if expr is a non negative number, and False otherwise.

    See NumberQ for details.
    "
FunctionQ::usage = "FunctionQ[expr]
    gives True if expr is a (non-pathological) function, and False otherwise.

    See http://tinyurl.com/yl43qrz for details."

(********************************** PLOT **************************************)
FilterOpts::usage = "FilterOpts[optsList, optsNamesOrRulesList]
    filters opts OptionPattern with given options names list. The difference
    between this function and the standard FilterRules is that rather than
    taking and returning List of options it takes and returns Sequence of
    options. This makes it more user-friendly when just passing optional
    parameters further. E.g.
        myPlot[expr_, opts:OptionsPattern[]] :=
            Module[{ (* ... *) },
                plot = Plot[expr, FilterOpts[{opts}, Options[Plot]]];
                (* ... *)
                plot
            ]
    "
OptsSeq::usage = "OptsSeq[funName]
    is a wrapper for Options[funName] but returns Sequence instead of List."
OptsMinus::usage = "OptsMinus[optsList, optsNamesOrRulesList]
    removes from list of options pattern options named in optsNamesOrRulesList.
    When given a options pattern as a second argument only the rhs of rules is
    ignored."
OptsUnion::usage = "OptsUnion[{OptionsPattern[]}, {OptionsPattern[]}, ...]
    unions the options pattern lists by name in order preference, e.g.
        OptsUnion[{a->1},{a->2,b->1},{b->2}] == {a->1,b->1}
    "
OptsDefaults::usage = "OptsDefaults[optionsArg, opts]
    returns the opts suplemented with missing default values according to
    Options function argument optionsArg; e.g.
        OptsDefaults[Plot, Mesh -> Full]
    will return Plot function options with Mesh option equal to Full.
    "
optsNames::usage = "temp"



Begin["`Private`"] (* Begin Private Context *)
(********************************** MATH **************************************)
PlusList[list_List] := Fold[Plus, 0, list]
PadRows[a:{__List}] := PadRight[#, Max[Length /@ a], #] & /@ a
SingletonOrId[x_] := If[x[[0]]===List, x, {x}]
Pairs[l1_List, l2_List] := ZipLists[l1,l2]
ZipLists[x:{__List}] := Transpose[PadRows[x]]
ZipLists[x__List] := ZipLists[{x}]
ZipLists[x__] := ZipLists[SingletonOrId/@{x}]

(********************************** TEST **************************************)
PositiveQ[expr_] := expr > 0
NonNegativeQ[expr_] := expr >= 0
PositiveIntegerQ[expr_] := IntegerQ[expr] && PositiveQ[expr]
NonNegativeIntegerQ[expr_] := IntegerQ[expr] && NonNegativeQ[expr]
PositiveNumberQ[expr_] := NumberQ[expr] && PositiveQ[expr]
NonNegativeNumberQ[expr_] := NumberQ[expr] && NonNegativeQ[expr]
FunctionQ[expr_] := (Head[expr]===Function) || (Head[expr] === Symbol &&
        ((DownValues[expr] =!= {}) ||
          (Intersection[Attributes[expr],{NumericFunction}] =!= {})
        ))


(********************************** OPTS **************************************)
optsNames[optsList:{OptionsPattern[]}] :=
    Cases[optsList, ((n_ :> _) | (n_ -> _) ) -> n]



FilterOpts[optsList:{OptionsPattern[]}, optsNamesOrRulesFilterList_] :=
    Module[{filtered},
        filtered = FilterRules[optsList, optsNamesOrRulesFilterList];
        filtered[[0]] = Sequence;
        filtered
    ]

OptsSeq[funName_] :=
  Module[{opts},
    opts = Options[funName];
    opts[[0]] = Sequence;
    opts
  ]

OptsMinus[optsList:{OptionsPattern[]}, {}] :=
  optsList
OptsMinus[optsList:{OptionsPattern[]}, optsNamesList:{__Symbol}] :=
  Cases[optsList, Except[(Fold[Alternatives, False, optsNamesList]) -> _]]
OptsMinus[optsList1:{OptionsPattern[]}, optsList2:{OptionsPattern[]}] :=
  OptsMinus[optsList1,optsNames[optsList2]]

OptsUnion[optsList1:{OptionsPattern[]}, optsList2:{OptionsPattern[]}] :=
  Union[optsList1,OptsMinus[optsList2, optsList1]]
OptsUnion[optsLists:Sequence[OptionsPattern[]]] :=
  Fold[OptsUnion,{},{optsLists}]

OptsDefaults[optionsArg_, opts___?OptionQ] :=
    OptsUnion[Flatten[{opts}],Options[optionsArg]]

End[] (* End Private Context *)

EndPackage[]
