/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.register.thread;

import java.io.File;
import java.util.Arrays;
import java.util.List;
import org.apache.log4j.Logger;

import pl.edu.mimuw.bioputer.tav4sb.register.Computation;
import pl.edu.mimuw.bioputer.tav4sb.register.Register;
import pl.edu.mimuw.bioputer.tav4sb.utils.MailSender;

/**
 *
 * @author michal
 */
public class Sender implements Runnable {

    private static final Logger logger = Logger.getLogger(Sender.class);
    private final String hostName;
    private final String port;
    private final String login;
    private final String password;
    private final MailSender sender;
    private final String from;
    private final String signature;
    private final int sendEveryMs;

    public Sender(String hostName, String port, String login, String password,
            String starttls, String from, String signature, int sendEveryMs) {
        this.hostName = hostName;
        this.port = port;
        this.login = login;
        this.password = password;
        this.sender = new MailSender(hostName, port, login, password, starttls);
        logger.debug("host name: " + getHostName());
        logger.debug("port: " + getPort());
        logger.debug("login: " + getLogin());
        logger.debug("password: " + getPassword());
        this.from = from;
        this.signature = signature;
        logger.debug("from: " + getFrom());
        logger.debug("signature: " + getSignature());
        this.sendEveryMs = sendEveryMs;
        logger.debug("send messages every " + getSendEveryMs()
                + " milliseconds.");
    }

    private MailSender getSender() {
        return sender;
    }

    public void run() {
        logger.debug("begin");
        try {
            while (true) {
                List<Computation> list = Register.getInstance()
                        .popAllCompletedComputation();
                logger.debug("sending " + list.size() + " computations");
                for (Computation c : list) {
                    try {
                        send(c);
                    } catch (Exception ex) {
                        logger.error(ex);
                    }
                }
                Thread.sleep(getSendEveryMs());
            }
        } catch (InterruptedException ex) {
            logger.error("interruption", ex);
        }
        logger.debug("stop");
    }

    public void send(Computation c) throws Exception {
        StringBuffer sb = new StringBuffer();
        sb.append("[STATS]\n\n");
        sb.append(c.getStatistic());
        sb.append("\n\n[RESULT]\n\n");
        sb.append("Please find attached.\n\n");
        sb.append(getSignature());
        String message = sb.toString();
        File[] attachments = { c.getResultFile() };
        logger.debug("Sending message to " + c.getEmail() + ":\n" + message
                + "\n\n with attachments:" + Arrays.toString(attachments));
        getSender().send(
                c.getEmail(),
                getFrom(),
                "[tav4sb] Results of the " + c.getComputationType()
                        + " computation.", message, attachments);
    }

    /**
     * @return the hostName
     */
    public String getHostName() {
        return hostName;
    }

    /**
     * @return the port
     */
    public String getPort() {
        return port;
    }

    /**
     * @return the login
     */
    public String getLogin() {
        return login;
    }

    /**
     * @return the password
     */
    public String getPassword() {
        return password;
    }

    public String getFrom() {
        return from;
    }

    public String getSignature() {
        return signature;
    }

    public int getSendEveryMs() {
        return sendEveryMs;
    }
}
